@@ -1,26 +1,24 @@
-r""" test_Ccp_Util.py - Parse, Query, Build, and Modify IOS-style configs
+r"""test_Ccp_Util.py - Parse, Query, Build, and Modify IOS-style configs
 
-     Copyright (C) 2021-2023 David Michael Pennington
-     Copyright (C) 2020-2021 David Michael Pennington at Cisco Systems
-     Copyright (C) 2019      David Michael Pennington at ThousandEyes
-     Copyright (C) 2014-2019 David Michael Pennington at Samsung Data Services
+Copyright (C) 2014-2025  David Michael Pennington
 
-     This program is free software: you can redistribute it and/or modify
-     it under the terms of the GNU General Public License as published by
-     the Free Software Foundation, either version 3 of the License, or
-     (at your option) any later version.
+This program is free software: you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
-     This program is distributed in the hope that it will be useful,
-     but WITHOUT ANY WARRANTY; without even the implied warranty of
-     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-     GNU General Public License for more details.
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
-     You should have received a copy of the GNU General Public License
-     along with this program.  If not, see <http://www.gnu.org/licenses/>.
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-     If you need to contact the author, you can do so by emailing:
-     mike [~at~] pennington [/dot\] net
+If you need to contact the author, you can do so by emailing:
+mike [~at~] pennington [/dot\] net
 """
+
 ##############################################################################
 # Disable SonarCloud warnings in this file
 #   - S1192: Define a constant instead of duplicating this literal
@@ -29,30 +27,74 @@ r""" test_Ccp_Util.py - Parse, Query, Build, and Modify IOS-style configs
 #   - S5852: Slow regex are security-sensitive
 #   - S6395: Unwrap this unnecessarily grouped regex subpattern.
 ##############################################################################
-#pragma warning disable S1192
-#pragma warning disable S1313
-#pragma warning disable S5843
-#pragma warning disable S5852
-#pragma warning disable S6395
+# pragma warning disable S1192
+# pragma warning disable S1313
+# pragma warning disable S5843
+# pragma warning disable S5852
+# pragma warning disable S6395
 
 
-import sys
+import ipaddress
 import os
+import pickle
+import sys
+from ipaddress import IPv4Address, IPv4Network, IPv6Address, IPv6Network
+import importlib.util
+
+import pytest
+from ciscoconfparse2.ccp_util import (
+    _RGX_IPV4ADDR,
+    _RGX_IPV6ADDR,
+    CiscoIOSInterface,
+    CiscoIOSXRInterface,
+    CiscoRange,
+    EUI64Obj,
+    IPv4Obj,
+    IPv6Obj,
+    L4Object,
+    MACObj,
+)
+from ciscoconfparse2.ccp_util import collapse_addresses as ccp_collapse_addresses
+from ciscoconfparse2.ccp_util import ip_factory
+from loguru import logger
+from macaddress import EUI48, EUI64, MAC, OUI
+
+from hypothesis import given, strategies, reproduce_failure
 
 sys.path.insert(0, "..")
 
-from ciscoconfparse.ccp_util import dns_lookup, reverse_dns_lookup, collapse_addresses
-from ciscoconfparse.ccp_util import IPv6Obj, IPv4Obj, L4Object, ip_factory
-from ciscoconfparse.ccp_util import _RGX_IPV4ADDR, _RGX_IPV6ADDR
-from ciscoconfparse.ccp_util import CiscoInterface, CiscoRange
-import pytest
 
-from loguru import logger
-from ipaddress import IPv4Network, IPv6Network, IPv4Address, IPv6Address
-import ipaddress
+def hypothesis_library_missing():
+    """Return True if hypothesis is missing"""
+    return importlib.util.find_spec("hypothesis") is None
+
+
+def testValues_pickle_01():
+    """Ensure that pickle() accepts an IPv4Obj() instance and saves a file"""
+
+    addr = IPv4Obj("192.0.2.1/24")
+
+    # Save to disk as a pickle
+    filename = "fixtures/plain_text/_test_01.pkl"
+    with open(filename, "wb") as fh:
+        pickle.dump(addr, fh)
+
+    # Load pickle from disk...
+    addr = pickle.load(open(filename, "rb"))
+
+    assert isinstance(addr, IPv4Obj)
+
+    os.remove(filename)
+
 
 @pytest.mark.parametrize(
-    "addr", ["192.0.2.1", "4.2.2.2", "10.255.255.255", "127.0.0.1",]
+    "addr",
+    [
+        "192.0.2.1",
+        "4.2.2.2",
+        "10.255.255.255",
+        "127.0.0.1",
+    ],
 )
 def test_IPv4_REGEX(addr):
     test_result = _RGX_IPV4ADDR.search(addr)
@@ -137,13 +179,13 @@ def testL4Object_asa_range01():
 def testL4Object_asa_lt01():
     pp = L4Object(protocol="tcp", port_spec="lt echo", syntax="asa")
     assert pp.protocol == "tcp"
-    assert pp.port_list ==sorted(range(1, 7))
+    assert pp.port_list == sorted(range(1, 7))
 
 
 def testL4Object_asa_gt01():
     pp = L4Object(protocol="tcp", port_spec="gt 65534", syntax="asa")
     assert pp.protocol == "tcp"
-    assert pp.port_list ==[65535]
+    assert pp.port_list == [65535]
 
 
 @pytest.mark.xfail(
@@ -156,6 +198,32 @@ def testL4Object_asa_lt02():
     assert pp.port_list == sorted(range(1, 7))
 
 
+@pytest.mark.skipif(
+    hypothesis_library_missing(), reason="hypothesis library is missing"
+)
+@given(
+    strategies.ip_addresses(),  # random_addr
+    strategies.integers(min_value=1, max_value=32),  # random_v4_mask
+    strategies.integers(min_value=1, max_value=128),  # random_v6_mask
+)
+def test_IPv4Obj_IPv6Obj_hypothesis(random_addr, random_v4_mask, random_v6_mask):
+    """Use hypothesis to test random IPv4 addresses."""
+
+    # random_addr could be either v4 or v6
+    try:
+        uut4 = IPv4Obj(f"{random_addr}/{random_v4_mask}")
+        _ = uut4.as_binary_tuple
+        assert uut4.as_cidr_addr == f"{random_addr}/{random_v4_mask}"
+        _ = uut4.as_cidr_net
+        _ = uut4.network
+    except (ValueError, ipaddress.AddressValueError):
+        uut6 = IPv6Obj(f"{random_addr}/{random_v6_mask}")
+        _ = uut6.as_binary_tuple
+        assert uut6.as_cidr_addr == f"{random_addr}/{random_v6_mask}"
+        _ = uut6.as_cidr_net
+        _ = uut6.network
+
+
 def testIPv4Obj_contains_01():
     ## Test ccp_util.IPv4Obj.__contains__()
     ##
@@ -183,7 +251,8 @@ def testIPv4Obj_contains_02():
 
 
 @pytest.mark.parametrize(
-    "addr_mask", [
+    "addr_mask",
+    [
         "1.0.0.1/24",
         "1.0.0.1/32",
         "1.0.0.1 255.255.255.0",
@@ -200,7 +269,7 @@ def testIPv4Obj_contains_02():
         "1.0.0.1 255.255.255.255",
         "1.0.0.1/255.255.255.0",
         "1.0.0.1/255.255.255.255",
-    ]
+    ],
 )
 def testIPv4Obj_parse(addr_mask):
     ## Ensure that IPv4Obj can correctly parse various inputs
@@ -235,17 +304,20 @@ def testIPv4Obj_set_masklen01():
         assert test_object.prefixlen == result_correct_masklen
         assert test_object.prefixlength == result_correct_masklen
 
+
 def testIPv4Obj_network_offset():
     test_object = IPv4Obj("192.0.2.28/24")
     assert test_object.masklength == 24
     assert test_object.network_offset == 28
 
+
 def testIPv4Obj_set_network_offset():
     test_object = IPv4Obj("192.0.2.28/24")
     # Change the last octet to be 200...
     test_object.network_offset = 200
     assert test_object == IPv4Obj("192.0.2.200/24")
 
+
 def testIPv4Obj_attributes_01():
     ## Ensure that attributes are accessible and pass the smell test
     test_object = IPv4Obj("1.0.0.1 255.255.255.0")
@@ -287,19 +359,28 @@ def test_ip_factory_inputs_01():
     test_params = (
         # Test format...
         #    (<dict with test inputs>, result_correct)
-        ({'val': '1.1.1.1/16', 'stdlib': False}, IPv4Obj("1.1.1.1/16")),
-        ({'val': '1.1.1.1/16', 'stdlib': True},  IPv4Network("1.1.1.1/16", strict=False)),
-        ({'val': '1.1.1.1/32', 'stdlib': False}, IPv4Obj("1.1.1.1/32")),
-        ({'val': '1.1.1.1/32', 'stdlib': True},  IPv4Address("1.1.1.1")),
-        ({'val': '2b00:cd80:14:10::1/64', 'stdlib': False}, IPv6Obj("2b00:cd80:14:10::1/64")),
-        ({'val': '2b00:cd80:14:10::1/64', 'stdlib': True}, IPv6Network("2b00:cd80:14:10::/64", strict=True)),
-        ({'val': '::1/64', 'stdlib': False}, IPv6Obj("::1/64")),
-        ({'val': '::1/64', 'stdlib': True},  IPv6Network("::0/64")),
-        ({'val': '::1/128', 'stdlib': False}, IPv6Obj("::1/128")),
-        ({'val': '::1/128', 'stdlib': True},  IPv6Address("::1")),
-        )
+        ({"val": "1.1.1.1/16", "stdlib": False}, IPv4Obj("1.1.1.1/16")),
+        (
+            {"val": "1.1.1.1/16", "stdlib": True},
+            IPv4Network("1.1.1.1/16", strict=False),
+        ),
+        ({"val": "1.1.1.1/32", "stdlib": False}, IPv4Obj("1.1.1.1/32")),
+        ({"val": "1.1.1.1/32", "stdlib": True}, IPv4Address("1.1.1.1")),
+        (
+            {"val": "2b00:cd80:14:10::1/64", "stdlib": False},
+            IPv6Obj("2b00:cd80:14:10::1/64"),
+        ),
+        (
+            {"val": "2b00:cd80:14:10::1/64", "stdlib": True},
+            IPv6Network("2b00:cd80:14:10::/64", strict=True),
+        ),
+        ({"val": "::1/64", "stdlib": False}, IPv6Obj("::1/64")),
+        ({"val": "::1/64", "stdlib": True}, IPv6Network("::0/64")),
+        ({"val": "::1/128", "stdlib": False}, IPv6Obj("::1/128")),
+        ({"val": "::1/128", "stdlib": True}, IPv6Address("::1")),
+    )
     for test_args, result_correct in test_params:
-        assert ip_factory(**test_args)==result_correct
+        assert ip_factory(**test_args) == result_correct
 
 
 def test_ip_factory_inputs_02():
@@ -353,6 +434,7 @@ def test_ip_factory_inputs_02():
     with pytest.raises(ipaddress.AddressValueError):
         ip_factory("FE80:AAAA::DEAD:BEEEEEEEEEEF", stdlib=False, mode="auto_detect")
 
+
 def testIPv6Obj_attributes_01():
     ## Ensure that attributes are accessible and pass the smell test
     test_object = IPv6Obj("2001::dead:beef/64")
@@ -397,16 +479,19 @@ def testIPv6Obj_attributes_01():
     for attribute, result_correct in results_correct:
 
         assert getattr(test_object, attribute) == result_correct
- 
+
+
 def testIPv6Obj_network_offset_01():
     test_object = IPv6Obj("2001::dead:beef/64")
     assert test_object.network_offset == 3735928559
 
+
 def testIPv6Obj_set_network_offset_01():
     test_object = IPv6Obj("2001::dead:beef/64")
     test_object.network_offset = 200
     assert test_object == IPv6Obj("2001::c8/64")
 
+
 def testIPv4Obj_sort_01():
     """Simple IPv4Obj sorting test"""
     cidr_addrs_list = [
@@ -479,7 +564,7 @@ def testIPv4Obj_recursive():
 
 
 def testIPv4Obj_from_int():
-    assert IPv4Obj(2886729984).ip == IPv4Address('172.16.1.0')
+    assert IPv4Obj(2886729984).ip == IPv4Address("172.16.1.0")
 
 
 def testIPv4Obj_neq_01():
@@ -572,6 +657,11 @@ def testIPv6Obj_recursive():
     assert obj.prefixlen == 64
 
 
+def testIPv6Obj_v4_embedded_in_v6_cidr():
+    """Ensure that IPv4 mapped in IPv6 as CIDR parses"""
+    assert IPv6Obj("::ffff:0.0.0.0/1").as_cidr_net == "::/1"
+
+
 def testIPv6Obj_neq_01():
     """Simple in-equality test fail (ref - Github issue #180)"""
     assert IPv6Obj("::1") != ""
@@ -597,53 +687,159 @@ def testIPv6Obj_lt_01():
     assert IPv6Obj("::1") < IPv6Obj("::2")
 
 
+def testIPv6Obj_IPv4_embedded_in_IPv6_01():
+    """Test IPv6Obj with an IPv4 address (192.168.1.254) embedded in an IPv6 address"""
+    assert IPv6Obj("::192.168.1.254") == IPv6Obj("::c0a8:1fe")
+
+
+def testIPv6Obj_IPv4_embedded_in_IPv6_02():
+    """Test IPv6Obj with an IPv4 address (192.0.2.33) embedded in an IPv6 address"""
+    assert IPv6Obj("2001:db8:122:344::192.0.2.33") == IPv6Obj(
+        "2001:db8:122:344::c000:221"
+    )
+
+
+def testIPv6Obj_IPv4_embedded_in_IPv6_03():
+    """Test IPv6Obj with an RFC 6052 NAT64 prefix (64:ff9b::) using IPv4 address (10.20.0.1) embedded in an IPv6 address"""
+    assert IPv6Obj("64:ff9b::192.0.2.33") == IPv6Obj("64:ff9b::c000:221")
+
+
+def testIPv6Obj_IPv4_embedded_in_IPv6_04():
+    """Test IPv6Obj with an IPv4 address (192.0.2.4) embedded in an IPv6 address"""
+    assert IPv6Obj("::ffff:192.0.2.4") == IPv6Obj("::ffff:c000:204")
+
+
+def testIPv6Obj_IPv6_error_01():
+    """Test IPv6Obj with an invalid address"""
+    with pytest.raises(ValueError):
+        IPv6Obj("hello,2001:db8::1/127")
+
+
+def testMACObj_int_01():
+    assert int(MACObj("0001.dead.beef")) == 8030895855
+
+
+def testMACObj_oui_01():
+    assert MACObj("0001.dead.beef").oui == OUI("00-01-DE")
+
+
+def testMACObj_cisco_01():
+    assert MACObj("0001.dead.beef").cisco == "0001.dead.beef"
+
+
+def testMACObj_cisco_02():
+    assert MACObj("00-01-de-ad-be-ef").cisco == "0001.dead.beef"
+
+
+def testMACObj_cisco_03():
+    assert MACObj("00:01:de:ad:be:ef").cisco == "0001.dead.beef"
+
+
+def testMACObj_cisco_04():
+    assert MACObj("0001deadbeef").cisco == "0001.dead.beef"
+
+
+def testMACObj_dash():
+    assert MACObj("0001.dead.beef").dash == "00-01-de-ad-be-ef"
+
+
+def testMACObj_colon():
+    assert MACObj("0001.dead.beef").colon == "00:01:de:ad:be:ef"
+
+
+def testMACObj_invalid_01():
+    with pytest.raises(ValueError):
+        assert MACObj("0001.dead")
+
+
+def testMACObj_invalid_02():
+    """This test is to check for an invalid MAC address format"""
+    with pytest.raises(ValueError):
+        assert MACObj("0001.dead.dead.dead.dead.dead.beef")
+
+
+def testMACObj_equality_01():
+    """Ensure that the same mac addresses in two instances test as equal"""
+    assert MACObj("dead.beef.0001") == MACObj("dead.beef.0001")
+
+
+def testMACObj_equality_02():
+    """Ensure that the same mac addresses in two instances (with different input formats) test as equal"""
+    assert MACObj("dead.beef.0001") == MACObj("de-ad-be-ef-00-01")
+
+
+def testMACObj_equality_03():
+    """Ensure that the same mac addresses in two instances (with different input formats) test as equal"""
+    assert MACObj("dead.beef.0001") == MACObj("de-ad-BE-EF-00-01")
+
+
+def testMACObj_equality_04():
+    """Ensure that the same mac addresses in two instances (comparing MACObj() and EUI48()) test as equal"""
+    assert MACObj("dead.beef.0001") == EUI48("de-ad-be-ef-00-01")
+
+
+def testMACObj_equality_05():
+    """Ensure we can compare to native macaddress instance types (i.e. macaddress.MAC)"""
+    assert MACObj("dead.beef.0001") == MAC("de-ad-be-ef-00-01")
+
+
+def testMACObj_inequality_01():
+    """Ensure that different input mac addresses and different input objects test as not equal"""
+    assert MACObj("dead.beef.0001") != EUI48("ff-ff-ff-ff-ff-ff")
+
+
+def testEUI64Obj_dash_01():
+    """Test EUI64Obj.dash"""
+    assert EUI64Obj("0001.dead.beef.0001").dash == "00-01-de-ad-be-ef-00-01"
+
+
+def testEUI64Obj_colon_01():
+    """Test EUI64Obj.colon"""
+    assert EUI64Obj("0001.dead.beef.0001").colon == "00:01:de:ad:be:ef:00:01"
+
+
+def testEUI64Obj_equality_01():
+    """Ensure that the same eui64 addresses in two instances test as equal"""
+    assert EUI64Obj("0001.dead.beef.0001") == EUI64Obj("0001.dead.beef.0001")
+
+
+def testEUI64Obj_equality_02():
+    """Ensure that the same eui64 addresses in two instances (with different input formats) test as equal"""
+    assert EUI64Obj("0001.dead.beef.0001") == EUI64Obj("00-01-de-ad-be-ef-00-01")
+
+
+def testEUI64Obj_equality_03():
+    """Ensure that the same eui64 addresses in two instances (with different input formats) test as equal"""
+    assert EUI64Obj("0001.dead.beef.0001") == EUI64Obj("00-01-de-ad-BE-EF-00-01")
+
+
+def testEUI64Obj_equality_04():
+    """Ensure we can compare to native macaddress instance types (i.e. macaddress.EUI64)"""
+    assert EUI64Obj("0001.dead.beef.0001") == EUI64("00-01-de-ad-be-ef-00-01")
+    assert EUI64Obj("0001.dead.beef.0001") != EUI64("ff-ff-ff-ff-ff-ff-ff-ff")
+
+
 def test_collapse_addresses_01():
-
-    net_collapsed = ipaddress.collapse_addresses([IPv4Network('192.0.0.0/22'), IPv4Network('192.0.2.128/25')])
-
-
+    """Test ipaddress.collapse_addresses() can collapse a list of IPv4Network objects with ipaddress.collapse_addresses()"""
+    net_collapsed = ipaddress.collapse_addresses(
+        [IPv4Network("192.0.0.0/22"), IPv4Network("192.0.2.128/25")]
+    )
     for idx, entry in enumerate(net_collapsed):
-        if idx==0:
+        if idx == 0:
             assert entry == IPv4Network("192.0.0.0/22")
 
 
 def test_collapse_addresses_02():
-    net_list = [IPv4Obj('192.0.2.128/25'), IPv4Obj('192.0.0.0/26')]
-    collapsed_list = sorted(collapse_addresses(net_list))
-    assert collapsed_list[0].network_address==IPv4Obj('192.0.0.0/26').ip
-    assert collapsed_list[1].network_address==IPv4Obj('192.0.2.128/25').ip
+    """Ensure that the native ccp_util.collapse_addresses can collapse a list of IPv4Obj objects into native ipaddress objects"""
+    net_list = [IPv4Obj("192.0.2.128/25"), IPv4Obj("192.0.0.0/26")]
+    collapsed_list = sorted(ccp_collapse_addresses(net_list))
+    assert collapsed_list[0].network_address == IPv4Obj("192.0.0.0/26").ip
+    assert collapsed_list[1].network_address == IPv4Obj("192.0.2.128/25").ip
 
 
-def test_dns_lookup():
-    # Use my hostname to test...
-    test_hostname = "pennington.net"
-    result_correct_address = "65.19.187.2"
-    result_correct = {"addrs": [result_correct_address], "name": test_hostname, "error": "", "record_type": "A"}
-    try:
-        test_result = dns_lookup(test_hostname, timeout=0.45)
-    except Exception as eee:
-        error = f"Skipping test_dns_lookup due to Exception: {eee}"
-        logger.warning(error)
-        pytest.skip(error)
-
-    if test_result["error"] != "":
-        assert dns_lookup(test_hostname) == result_correct
-    else:
-        pytest.skip(test_result["error"])
-
-
-def test_reverse_dns_lookup():
-
-    #result_correct = {"addrs": ["127.0.0.1"], "name": "localhost.", "error": ""}
-    try:
-        test_result = reverse_dns_lookup("127.0.0.1", timeout=0.1)
-        assert test_result["error"] == ""
-    except Exception:
-        pytest.skip(test_result["error"])
-
-def test_CiscoInterface_01():
-    """Check that a single number is parsed correctly"""
-    uut = CiscoInterface("Ethernet1")
+def test_CiscoIOSInterface_01():
+    """Check that a single interface number is parsed correctly"""
+    uut = CiscoIOSInterface("Ethernet1")
     assert uut.prefix == "Ethernet"
     assert uut.card is None
     assert uut.slot is None
@@ -652,9 +848,10 @@ def test_CiscoInterface_01():
     assert uut.channel is None
     assert uut.interface_class is None
 
-def test_CiscoInterface_02():
-    """Check that a card and port is parsed correctly"""
-    uut = CiscoInterface("Ethernet1/42")
+
+def test_CiscoIOSInterface_02():
+    """Check that an interface card and port is parsed correctly"""
+    uut = CiscoIOSInterface("Ethernet1/42")
     assert uut.prefix == "Ethernet"
     assert uut.slot == 1
     assert uut.card is None
@@ -663,9 +860,10 @@ def test_CiscoInterface_02():
     assert uut.channel is None
     assert uut.interface_class is None
 
-def test_CiscoInterface_03():
-    """Check that a card and large port-number is parsed correctly"""
-    uut = CiscoInterface("Ethernet1/4242")
+
+def test_CiscoIOSInterface_03():
+    """Check that an interface card and large port-number is parsed correctly"""
+    uut = CiscoIOSInterface("Ethernet1/4242")
     assert uut.prefix == "Ethernet"
     assert uut.slot == 1
     assert uut.card is None
@@ -674,9 +872,10 @@ def test_CiscoInterface_03():
     assert uut.channel is None
     assert uut.interface_class is None
 
-def test_CiscoInterface_04():
-    """Check that a card, port and subinterface is parsed correctly"""
-    uut = CiscoInterface("Ethernet1/42.5")
+
+def test_CiscoIOSInterface_04():
+    """Check that an interface card, port and subinterface is parsed correctly"""
+    uut = CiscoIOSInterface("Ethernet1/42.5")
     assert uut.prefix == "Ethernet"
     assert uut.slot == 1
     assert uut.card is None
@@ -685,9 +884,10 @@ def test_CiscoInterface_04():
     assert uut.channel is None
     assert uut.interface_class is None
 
-def test_CiscoInterface_05():
-    """Check that a card, slot, port  is parsed correctly"""
-    uut = CiscoInterface("Ethernet1/3/42")
+
+def test_CiscoIOSInterface_05():
+    """Check that an interface card, slot, port is parsed correctly"""
+    uut = CiscoIOSInterface("Ethernet1/3/42")
     assert uut.prefix == "Ethernet"
     assert uut.slot == 1
     assert uut.card == 3
@@ -696,9 +896,10 @@ def test_CiscoInterface_05():
     assert uut.channel is None
     assert uut.interface_class is None
 
-def test_CiscoInterface_06():
-    """Check that a card, slot, port and subinterface  is parsed correctly"""
-    uut = CiscoInterface("Ethernet1/3/42.5")
+
+def test_CiscoIOSInterface_06():
+    """Check that an interface card, slot, port and subinterface is parsed correctly"""
+    uut = CiscoIOSInterface("Ethernet1/3/42.5")
     assert uut.prefix == "Ethernet"
     assert uut.slot == 1
     assert uut.card == 3
@@ -707,9 +908,10 @@ def test_CiscoInterface_06():
     assert uut.channel is None
     assert uut.interface_class is None
 
-def test_CiscoInterface_07():
+
+def test_CiscoIOSInterface_07():
     """Check that a card, slot, port, subinterface, and channel is parsed correctly"""
-    uut = CiscoInterface("Ethernet1/3/42.5:9")
+    uut = CiscoIOSInterface("Ethernet1/3/42.5:9")
     assert uut.prefix == "Ethernet"
     assert uut.slot == 1
     assert uut.card == 3
@@ -718,19 +920,21 @@ def test_CiscoInterface_07():
     assert uut.channel == 9
     assert uut.interface_class is None
 
-def test_CiscoInterface_08():
+
+def test_CiscoIOSInterface_08():
     """Check that a card, slot, port, subinterface, and channel is parsed correctly from a dict"""
-    uut = CiscoInterface(
+    uut = CiscoIOSInterface(
         interface_dict={
-            'prefix': 'Ethernet',
-            'card': 2,
-            'slot': 1,
-            'port': 3,
-            'digit_separator': '/',
-            'subinterface': 4,
-            'channel': 5,
-            'interface_class': None,
-        })
+            "prefix": "Ethernet",
+            "card": 2,
+            "slot": 1,
+            "port": 3,
+            "digit_separator": "/",
+            "subinterface": 4,
+            "channel": 5,
+            "interface_class": None,
+        }
+    )
     assert uut.prefix == "Ethernet"
     assert uut.slot == 1
     assert uut.card == 2
@@ -740,31 +944,34 @@ def test_CiscoInterface_08():
     assert uut.digit_separator == "/"
     assert uut.interface_class is None
 
-def test_CiscoInterface_09():
+
+def test_CiscoIOSInterface_09():
     """Check that a port is parsed correctly from a dict"""
-    uut = CiscoInterface(
+    uut = CiscoIOSInterface(
         interface_dict={
-            'prefix': 'Ethernet',
-            'card': None,
-            'slot': None,
-            'port': 1,
-            'digit_separator': None,
-            'subinterface': None,
-            'channel':None,
-            'interface_class': None,
-        })
+            "prefix": "Ethernet",
+            "card": None,
+            "slot": None,
+            "port": 1,
+            "digit_separator": None,
+            "subinterface": None,
+            "channel": None,
+            "interface_class": None,
+        }
+    )
     assert uut.prefix == "Ethernet"
     assert uut.slot == None
     assert uut.card is None
-    assert uut.port is 1
+    assert uut.port == 1
     assert uut.subinterface is None
     assert uut.channel is None
     assert uut.digit_separator is None
     assert uut.interface_class is None
 
-def test_CiscoInterface_10():
+
+def test_CiscoIOSInterface_10():
     """Check that a card, slot, port, subinterface, and channel is parsed correctly"""
-    uut = CiscoInterface("Serial1/3/42.5:9 multipoint")
+    uut = CiscoIOSInterface("Serial1/3/42.5:9 multipoint")
     assert uut.prefix == "Serial"
     assert uut.slot == 1
     assert uut.card == 3
@@ -773,48 +980,62 @@ def test_CiscoInterface_10():
     assert uut.channel == 9
     assert uut.interface_class == "multipoint"
 
+
 def test_CiscoRange_01():
     """Basic vlan range test"""
     result_correct = {1, 2, 3}
     uut_str = "1-3"
-    assert CiscoRange(uut_str, result_type=int).as_set(result_type=int) == result_correct
+    assert (
+        CiscoRange(uut_str, result_type=int).as_set(result_type=int) == result_correct
+    )
 
 
 def test_CiscoRange_02():
     """Basic vlan range test"""
     result_correct = {1, 3}
     uut_str = "1,3"
-    assert CiscoRange(uut_str, result_type=int).as_set(result_type=int) == result_correct
+    assert (
+        CiscoRange(uut_str, result_type=int).as_set(result_type=int) == result_correct
+    )
 
 
 def test_CiscoRange_03():
     """Basic vlan range test"""
     result_correct = {1, 2, 3, 4, 5}
     uut_str = "1,2-4,5"
-    assert CiscoRange(uut_str, result_type=int).as_set(result_type=int) == result_correct
+    assert (
+        CiscoRange(uut_str, result_type=int).as_set(result_type=int) == result_correct
+    )
 
 
 def test_CiscoRange_04():
     """Basic vlan range test"""
     result_correct = {1, 2, 3, 4, 5}
     uut_str = "1-3,4,5"
-    assert CiscoRange(uut_str, result_type=int).as_set(result_type=int) == result_correct
+    assert (
+        CiscoRange(uut_str, result_type=int).as_set(result_type=int) == result_correct
+    )
 
 
 def test_CiscoRange_05():
     """Basic vlan range test"""
     result_correct = {1, 2, 3, 4, 5}
     uut_str = "1,2,3-5"
-    assert CiscoRange(uut_str, result_type=int).as_set(result_type=int) == result_correct
+    assert (
+        CiscoRange(uut_str, result_type=int).as_set(result_type=int) == result_correct
+    )
 
 
 def test_CiscoRange_06():
     """Basic slot range test"""
     result_correct = {"1/1", "1/2", "1/3", "1/4", "1/5"}
     uut_str = "1/1-3,4,5"
-    # the CiscoRange() result_type None is a CiscoInterface() type with a 
+    # the CiscoRange() result_type None is a CiscoIOSInterface() type with a
     #     port attribute...
-    assert CiscoRange(uut_str, result_type=None).as_set(result_type=str) == result_correct
+    assert (
+        CiscoRange(uut_str, result_type=CiscoIOSInterface).as_set(result_type=str)
+        == result_correct
+    )
     assert CiscoRange(uut_str).iterate_attribute == "port"
 
 
@@ -880,48 +1101,67 @@ def test_CiscoRange_15():
     uut_str = "Eth 2/1/1,2,3-5"
     assert CiscoRange(uut_str).as_set(result_type=str) == result_correct
 
+
 def test_CiscoRange_16():
     """Basic interface port range test"""
     result_correct = {"Eth7", "Eth8", "Eth9"}
     uut_str = "Eth 7-9"
     assert CiscoRange(uut_str).as_set(result_type=str) == result_correct
 
+
 def test_CiscoRange_17():
     """Basic interface slot and port range test"""
     result_correct = {"Eth1/7", "Eth1/8", "Eth1/9"}
     uut_str = "Eth 1/7-9"
     assert CiscoRange(uut_str).as_set(result_type=str) == result_correct
 
+
 def test_CiscoRange_18():
     """Basic interface slot, card, and port range test"""
     result_correct = {"Eth1/2/7", "Eth1/2/8", "Eth1/2/9"}
     uut_str = "Eth 1/2/7-9"
     assert CiscoRange(uut_str).as_set(result_type=str) == result_correct
 
+
 def test_CiscoRange_19():
     """Basic interface slot, card, port, and subinterface range test"""
     result_correct = {"Eth1/2/3.7", "Eth1/2/3.8", "Eth1/2/3.9"}
     uut_str = "Eth 1/2/3.7-9"
     assert CiscoRange(uut_str).as_set(result_type=str) == result_correct
 
+
 def test_CiscoRange_20():
     """Basic interface slot, card, port, subinterface and channel range test"""
     result_correct = {"Eth1/2/3.4:7", "Eth1/2/3.4:8", "Eth1/2/3.4:9"}
     uut_str = "Eth 1/2/3.4:7-9"
     assert CiscoRange(uut_str).as_set(result_type=str) == result_correct
 
+
 def test_CiscoRange_21():
     """Basic interface slot, card, port, subinterface, channel and interface_class range test"""
-    result_correct = {"Eth1/2/3.4:7 multipoint", "Eth1/2/3.4:8 multipoint", "Eth1/2/3.4:9 multipoint"}
+    result_correct = {
+        "Eth1/2/3.4:7 multipoint",
+        "Eth1/2/3.4:8 multipoint",
+        "Eth1/2/3.4:9 multipoint",
+    }
     uut_str = "Eth 1/2/3.4:7-9 multipoint"
     assert CiscoRange(uut_str).as_set(result_type=str) == result_correct
 
+
 def test_CiscoRange_22():
     """Parse a string with a common prefix on all of the CiscoRange() inputs"""
     result_correct = {
-        "Eth1/1", "Eth1/10", "Eth1/12", "Eth1/13", "Eth1/14",
-        "Eth1/15", "Eth1/16", "Eth1/17", "Eth1/18", "Eth1/19",
-        "Eth1/20"
+        "Eth1/1",
+        "Eth1/10",
+        "Eth1/12",
+        "Eth1/13",
+        "Eth1/14",
+        "Eth1/15",
+        "Eth1/16",
+        "Eth1/17",
+        "Eth1/18",
+        "Eth1/19",
+        "Eth1/20",
     }
     uut_str = "Eth1/1,Eth1/12-20,Eth1/16,Eth1/10"
     assert CiscoRange(uut_str).as_set(result_type=str) == result_correct
@@ -930,36 +1170,44 @@ def test_CiscoRange_22():
 def test_CiscoRange_23():
     """Check that the exact results are correct for CiscoRange().as_set() with a redundant input ('Eth1/1')"""
     result_correct = {
-        CiscoInterface("Eth1/1"),
-        CiscoInterface("Eth1/2"),
-        CiscoInterface("Eth1/3"),
-        CiscoInterface("Eth1/4"),
-        CiscoInterface("Eth1/5"),
-        CiscoInterface("Eth1/16"),
+        CiscoIOSInterface("Eth1/1"),
+        CiscoIOSInterface("Eth1/2"),
+        CiscoIOSInterface("Eth1/3"),
+        CiscoIOSInterface("Eth1/4"),
+        CiscoIOSInterface("Eth1/5"),
+        CiscoIOSInterface("Eth1/16"),
     }
     uut_str = "Eth1/1,Eth1/1-5,Eth1/16"
-    # CiscoRange(text="foo", result_type=None) returns CiscoInterface() instances...
-    assert CiscoRange(uut_str, result_type=None).as_set(result_type=None) == result_correct
+    # CiscoRange(text="foo", result_type=None) returns CiscoIOSInterface() instances...
+    assert (
+        CiscoRange(uut_str, result_type=None).as_set(result_type=None) == result_correct
+    )
+
 
 def test_CiscoRange_24():
     """Check that the exact results are correct for CiscoRange().as_list() with a redundant input ('Eth1/1')"""
     result_correct = [
-        CiscoInterface("Eth1/1"),
-        CiscoInterface("Eth1/2"),
-        CiscoInterface("Eth1/3"),
-        CiscoInterface("Eth1/4"),
-        CiscoInterface("Eth1/5"),
-        CiscoInterface("Eth1/16"),
+        CiscoIOSInterface("Eth1/1"),
+        CiscoIOSInterface("Eth1/2"),
+        CiscoIOSInterface("Eth1/3"),
+        CiscoIOSInterface("Eth1/4"),
+        CiscoIOSInterface("Eth1/5"),
+        CiscoIOSInterface("Eth1/16"),
     ]
     uut_str = "Eth1/1,Eth1/1-5,Eth1/16"
-    # CiscoRange(text="foo", result_type=None) returns CiscoInterface() instances...
-    assert CiscoRange(uut_str, result_type=None).as_list(result_type=None) == result_correct
+    # CiscoRange(text="foo", result_type=None) returns CiscoIOSInterface() instances...
+    assert (
+        CiscoRange(uut_str, result_type=None).as_list(result_type=None)
+        == result_correct
+    )
+
 
 def test_CiscoRange_compressed_str_01():
     """compressed_str test with a very basic set of vlan numbers"""
     uut_str = "1,2,911"
     assert CiscoRange(uut_str, result_type=int).as_compressed_str() == "1,2,911"
 
+
 def test_CiscoRange_compressed_str_02():
     """compressed_str test with vlan number ranges"""
     uut_str = "1,2, 3, 6, 7,  8 , 9, 911"
@@ -970,16 +1218,51 @@ def test_CiscoRange_contains_01():
     """Check that the exact results are correct that a CiscoRange() contains an input"""
     uut_str = "Ethernet1/1-20"
     # Ethernet1/5 is in CiscoRange("Ethernet1/1-20")...
-    assert CiscoInterface("Ethernet1/5") in CiscoRange(uut_str)
+    assert CiscoIOSInterface("Ethernet1/5") in CiscoRange(uut_str)
+
 
 def test_CiscoRange_contains_02():
     """Check that the exact results are correct that a CiscoRange() does not contain an input"""
     uut_str = "Ethernet1/1-20"
     # Ethernet1/5 is in CiscoRange("Ethernet1/1-20")...
-    assert (CiscoInterface("Ethernet1/48") in CiscoRange(uut_str)) is False
+    assert (CiscoIOSInterface("Ethernet1/48") in CiscoRange(uut_str)) is False
 
-#pragma warning restore S1192
-#pragma warning restore S1313
-#pragma warning restore S5843
-#pragma warning restore S5852
-#pragma warning restore S6395
+
+def test_CiscoRange_insert_01():
+    """Check that CiscoRange().insert(sort=False) works correctly"""
+    uut_str = "Ethernet1/1-20"
+    uut = CiscoRange(uut_str)
+
+    uut.insert(0, "Ethernet1/100", sort=False)
+
+    assert CiscoIOSInterface("Ethernet1/100") in uut.data
+    assert uut.data[0] == CiscoIOSInterface("Ethernet1/100")
+
+
+def test_CiscoRange_insert_02():
+    """Check that CiscoRange().insert(sort=True) works correctly"""
+    uut_str = "Ethernet1/1-20"
+    uut = CiscoRange(uut_str)
+
+    uut.insert(0, "Ethernet1/100", sort=True)
+
+    assert CiscoIOSInterface("Ethernet1/100") in uut.data
+    assert uut.data[-1] == CiscoIOSInterface("Ethernet1/100")
+
+
+def test_CiscoRange_insert_03():
+    """Check that CiscoRange().insert() works correctly and sorts by default"""
+    uut_str = "Ethernet1/1-20"
+    uut = CiscoRange(uut_str)
+
+    uut.insert(0, "Ethernet1/100")
+
+    assert CiscoIOSInterface("Ethernet1/100") in uut.data
+    assert uut.data[-1] == CiscoIOSInterface("Ethernet1/100")
+
+
+# pragma warning restore S1192
+# pragma warning restore S1313
+# pragma warning restore S5843
+# pragma warning restore S5852
+# pragma warning restore S6395
